############################################################################################################################
############ Setup dependencies ############################################################################################
############################################################################################################################

library(polynom)
require(sampling)
require(plyr)
require(phia)
closest<-function(xv,sv){xv[which(abs(xv-sv)==min(abs(xv-sv)))]}
is.nonzero <- function(x){length(x)==1 & is.numeric(x)}
co.var <- function(x) ( sd(x)/mean(x) )

confint.cv <- function(x,alpha=.05){                                          # Calculate the confidence interval of the Coefficient of Variation of the vector x
          x <- na.omit(x)                                                     # Original author: Kevin Wright
          v <- length(x)-1                                                    # See: Vangel, Mark.  1996. Confidence Intervals for a Normal Coefficient
          mu <- mean(x)                                                       # of Variation. American Statistician, Vol 15, No1, p. 21--26.
          sigma <- sqrt(var(x))
          k <- sigma/mu
            u1 <- qchisq(1-alpha/2,v)                 #Modified McKay method. See equation 16 in Vangel (1996).
            u2 <- qchisq(alpha/2,v)
            lower <- k/sqrt(( (u1+2)/(v+1) -1)*k*k + u1/v)
            upper <- k/sqrt(( (u2+2)/(v+1) -1)*k*k + u2/v)
            ci <- c(lower,upper)
            attr(ci,"CV") <- k
            attr(ci,"alpha") <- alpha
            return(ci)
          }
          
summarySE <- function(data=NULL, measurevar, groupvars=NULL, na.rm=FALSE, conf.interval=.95, .drop=TRUE) {
    require(plyr)
    length2 <- function (x, na.rm=FALSE) { if (na.rm) sum(!is.na(x)) else length(x) }
    datac <- ddply(data, groupvars, .drop=.drop,
                   .fun= function(xx, col, na.rm) {
                           c( N    = length2(xx[,col], na.rm=na.rm),
                              mean = mean   (xx[,col], na.rm=na.rm),
                              sd   = sd     (xx[,col], na.rm=na.rm),
                              mad   = mad     (xx[,col], na.rm=na.rm),
                              cv   = co.var     (xx[,col]),
                              cv.ci   = confint.cv     (xx[,col])
                              )
                          },
                    measurevar,
                    na.rm
                  )
    datac <- rename(datac, c("mean"=measurevar))
    datac$se <- datac$sd / sqrt(datac$N)  # Calculate standard error of the mean
    # Confidence interval multiplier for standard error
    # Calculate t-statistic for confidence interval:
    # e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
    ciMult <- qt(conf.interval/2 + .5, datac$N-1)
    datac$ci <- datac$se * ciMult
    return(datac)
}


############################################################################################################################
############ Read in data ##################################################################################################
############################################################################################################################

labdata = read.table("~ filepath /labdata_ecology.txt", header=T)
microcosmdata =  read.table("~ filepath /microcosmdata_ecology.txt", header=T)


############################################################################################################################
############ Common garden experiment, Relative Growth Rates ###############################################################
############################################################################################################################
refsize = log(1+0.8)         #set reference size
  laboutput = as.data.frame(unique(labdata$label))                                                             #create data frame for output
  names(laboutput) = c("label")
  laboutput$RGR = c("NA")
  laboutput$plateau = c("NA")
  laboutput$d = c("NA")                                                                                        
  laboutput$fam = as.factor(labdata$fam[match(laboutput$label,labdata$label)])
  laboutput$pop<-as.factor(labdata$pop[match(laboutput$label,labdata$label)])
  laboutput$spec = as.factor(labdata$spec[match(laboutput$label,labdata$label)])
  laboutput$temp = as.numeric(labdata$temp[match(laboutput$label,labdata$label)])
  n = length(laboutput$label)
  hwc<-NA
  spc<-NA
  for(i in 1:n){
        name = as.vector(laboutput$label[i])
        spec = as.vector(laboutput$spec[i])
        alldata = subset(labdata,label==name)
        spc[i] = spec
        if (sum(na.exclude(alldata$hw[alldata$day==0]))>0 & sum(na.exclude(alldata$hw[alldata$day==42]))>0 & sum(na.exclude(alldata$hw[alldata$day==84]))>0 & sum(na.exclude(alldata$hw[alldata$day==126]))>0){  #If all 4 values are ok
           m1 = lm(log(1+hw)~day+I(day^2)+I(day^3),data=alldata)
           coefsim = as.matrix(coef(m1))
           a = as.numeric(coefsim[1,1])
           b = as.numeric(coefsim[2,1])
           c = as.numeric(coefsim[3,1])
           d = as.numeric(coefsim[4,1])
           tLc = solve(polynomial(c(a-refsize,b,c,d)))   #Solve polynomial equation for the point in time when head width = reference size
           tLc = as.numeric(tLc[which(Im(tLc)==0)])       #Discart solutions with an non-zero imaginary component
           tLc = tLc[tLc<127 &tLc>0]                      #Discard solutions outside the range of day 0 to day 126
           laboutput$RGR[i] = 3*d*tLc^2+2*c*tLc+b         #Outputs the RGR = the slope of the fitted polynomial at day = tLc
           }
           else{}                                         #do nothing for individuals with <4 measurement events
           }
laboutput = subset(laboutput,RGR!="NA")
laboutput$RGR = as.numeric(laboutput$RGR)


 
 
############################################################################################################################
############ Microcosm experiment, Relative Growth Rates ###################################################################
############################################################################################################################

  microcosmdata$id.spec.day = paste(microcosmdata$id.spec,microcosmdata$day,sep=".")
  s = strata(microcosmdata,stratanames=c("id.spec","day"),size=rep(as.vector(with(subset(microcosmdata,day==126),tapply(hw,id.spec.day,length)))*1,4),method=c("srswr")) #resample with replacement N observation from each Microcosm : Species : Measurement event, where N is the number of surviving larvae at day 126
  sdata = getdata(microcosmdata,s)                                                                #extract resampled dataset
  sdata = sdata[order(sdata$spec,sdata$temp,sdata$treatm,sdata$id.spec,sdata$day,sdata$hw),]      #sort observations within each measurement event with respect to increasing size
  sdata$ind = 1                                                                                   #reset individual IDs
  uniq = unique(sdata$id.spec.day) #the number of unique observations                             #get the unique ID for each Microcosm : Species : Measurement event
  for (i in 1:length(uniq)){ sdata$ind[sdata$id.spec.day==uniq[i]] = seq(1:length(sdata$ind[sdata$id.spec.day==uniq[i]])) }   #For each Microcosm : Species : Measurement event, this assigns each of the N resampled obervations to "pseudoindividuals" i1, i2, i3 ... iN
  sdata$id.spec.ind =  paste(sdata$id.spec,sdata$ind,sep=".")                                     #Creates a grouping variable for each Microcosm : Species : Measurement event : Pseudoindividual, to be used for fitting growth curves
  compoutput = subset(sdata, day==0)                                                              #Creates a data frame for the outputs
  compoutput$RGR = c("NA")

  for (i in 1:length(unique(sdata$id.spec.ind))){                                                 #For each pseudoindividual, fits the growth equation (Eq. 2 in Main text)
    name = as.vector(compoutput$id.spec.ind[i])                                                   #Get the ID for pseudoindividual i
    indiv = subset(sdata,id.spec.ind==name)                                                       #Extract observations for pseudoindividual i
    if (dim(indiv)[1]==4){                                                                        #Double check to the data is OK
       m1 = lm(log(1+hw)~day+I(day^2)+I(day^3),data=indiv)                                        #Fit Eq. (2) for pseudoindividual i.
       as.matrix(coef(m1))->coefsim                                                               #Extract polynomial coefficients
       as.numeric(coefsim[1,1])->a
       as.numeric(coefsim[2,1])->b
       as.numeric(coefsim[3,1])->c
       as.numeric(coefsim[4,1])->d
       tLc = solve(polynomial(c(a-refsize,b,c,d)))                                                #Solve polynomial equation for the point in time when head width = reference size
       tLc<-as.numeric(tLc[which(Im(tLc)==0)])                                                    #Discart solutions with an non-zero imaginary component
       tLc<-tLc[tLc<127 &tLc>0]                                                                   #Discard solutions outside the range of day 0 to day 126
       compoutput$RGR[i] = 3*d*tLc^2+2*c*tLc+b                                                    #Outputs the RGR = the slope of the fitted polynomial at day = tLc
       }
       else{}
}
compoutput = subset(compoutput,RGR!="NA")
compoutput$RGR = as.numeric(compoutput$RGR)






############################################################################################################################
######## FIGURE 1 ##########################################################################################################
############################################################################################################################
LABMEANsummary = aggregate(laboutput[c("RGR")],by=laboutput[c("spec","temp","fam")],FUN=mean)         #Aggregate data from the common garden: average RGR for each Temperature x Family group
LABMEANsummary$RGR = log(LABMEANsummary$RGR)
LABMEANsummary$Kelvin<-LABMEANsummary$temp+273.15
LABMEANsummary$kT<- -1/(8.617332e-05*LABMEANsummary$Kelvin)

LABsummary = summarySE(LABMEANsummary,measurevar="RGR",groupvars=c("spec","temp"),na.rm=TRUE)         #Aggregate data from the common garden: average RGR for each Temperature x Species group
LABsummary$Kelvin<-LABsummary$temp+273.15
LABsummary$kT<- -1/(8.617332e-05*LABsummary$Kelvin)

COMPMEANsummary = aggregate(compoutput[c("RGR")],by=compoutput[c("spec","treatm","temp","id.spec")],FUN=mean)               #Aggregate data from the Microcosms experiment: average for each Species-within-Microcosms group
COMPMEANsummary$RGR = log(COMPMEANsummary$RGR)
COMPMEANsummary$temp.text = COMPMEANsummary$temp
COMPMEANsummary$temp = as.numeric(COMPMEANsummary$temp)
COMPMEANsummary$temp[COMPMEANsummary$temp.text=="lo"] = c(16.3)
COMPMEANsummary$temp[COMPMEANsummary$temp.text=="hi"] = c(21.5)
COMPMEANsummary$Kelvin<-COMPMEANsummary$temp+273.15
COMPMEANsummary$kT<- -1/(8.617332e-05*COMPMEANsummary$Kelvin)

COMPsummary = summarySE(COMPMEANsummary,measurevar="RGR",groupvars=c("spec","treatm","temp"),na.rm=TRUE)                     #Aggregate data from the Microcosms experiment: average for each Species x Temperature group
COMPsummary$temp.text = COMPsummary$temp
COMPsummary$temp = as.numeric(COMPsummary$temp)
COMPsummary$temp[COMPsummary$temp.text=="lo"] = c(16.3)
COMPsummary$temp[COMPsummary$temp.text=="hi"] = c(21.5)
COMPsummary$Kelvin<-COMPsummary$temp+273.15
COMPsummary$kT<- -1/(8.617332e-05*COMPsummary$Kelvin)


dev.new(width=8.7, height=3.3)
par(mfrow=c(1,3),mar=c(5,5,6,1))
#A)
with(subset(LABMEANsummary,spec=="arm"),plot(jitter(kT-0.02,0.2),RGR,pch=1,cex=1.6,ylim=c(-5.5,-3.75),xlim=c(-40.2, -39.3),main="",xlab="temperature (1/-kT, eV)",ylab="ln(RGR)",cex.lab=1.1, cex.axis=1.1))
title("Individual rearing",line=4)
with(subset(LABMEANsummary,spec=="arm"),abline(lm(RGR~kT),lty=2,col="black"))

with(subset(LABMEANsummary,spec=="pul"),points(jitter(kT+0.02,0.2),RGR,pch=16,cex=1.6,col="gray40"))
with(subset(LABMEANsummary,spec=="pul"),abline(lm(RGR~kT),lty=2,col="gray40"))
legend("bottomright",cex=1.3,pch=c(16,1),col=c("gray40","black"),c(expression(italic("C. pulchellum")),expression(italic("C. armatum"))),bty="n")
legend("topleft",c("A"),bty="n", cex=1.6, adj=c(2,-0.2))
axis(3,expression(paste("16",degree,"C"),paste("19",degree,"C"),paste("21",degree,"C")),at=c(-40.0916214241359, -39.6532370449893, -39.3840822033468))
xlab=expression(paste("16 (",degree,"C)"),paste("19 (",degree,"C)"),paste("21 (",degree,"C)"))

#B)
with(subset(COMPsummary,spec=="arm" & treatm=="uni"),plot(kT-0.02,RGR,pch=19,cex=1.6,ylim=c(-5.5,-3.75),xlim=c(-40.2, -39.3),main="",xlab="temperature (1/-kT, eV)",ylab="ln(RGR)",cex.lab=1.1, cex.axis=1.1))
title("Microcosm experiment",line=4)
with(subset(COMPsummary,spec=="arm" & treatm=="uni"),lines(kT-0.02,RGR,lty=1))
with(subset(COMPsummary,spec=="arm" & treatm=="uni"),arrows(kT-0.02,RGR-sd,kT-0.02,RGR+sd,length=0))

with(subset(COMPsummary,spec=="arm" & treatm=="mix"),points(kT,RGR,pch=19,col="gray40",cex=1.6))
with(subset(COMPsummary,spec=="arm" & treatm=="mix"),lines(kT,RGR,lty=1))
with(subset(COMPsummary,spec=="arm" & treatm=="mix"),arrows(kT,RGR-sd,kT,RGR+sd,length=0))

with(subset(LABsummary,spec=="arm" & temp!=19.5),points(kT+0.02,RGR,pch=1,cex=1.6,col="black"))
with(subset(LABMEANsummary,spec=="arm"),abline(lm(RGR~kT),lty=2,col="black"))
with(subset(LABsummary,spec=="arm" & temp!=19.5),arrows(kT+0.02,RGR-sd,kT+0.02,RGR+sd,length=0))
legend("bottomright",cex=1.3,pch=c(19,19,1),col=c("gray40","black","black"),c("Interspecific", "Intraspecific", "Individual"),bty="n")
legend("topleft",c("B"),bty="n", cex=1.6, adj=c(2,-0.2))
mtext(expression(italic("C. armatum")),3,padj=-2.1)
axis(3,expression(paste("16",degree,"C"),paste("21",degree,"C")),at=c(-40.0916214241359, -39.3840822033468))

#C)
with(subset(COMPsummary,spec=="pul" & treatm=="uni"),plot(kT-0.02,RGR,pch=19,cex=1.6,ylim=c(-5.5,-3.75),xlim=c(-40.2, -39.3),main="",xlab="temperature (1/-kT, eV)",ylab="ln(RGR)",cex.lab=1.1, cex.axis=1.1))
title("Microcosm experiment",line=4)
with(subset(COMPsummary,spec=="pul" & treatm=="uni"),lines(kT-0.02,RGR,lty=1))
with(subset(COMPsummary,spec=="pul" & treatm=="uni"),arrows(kT-0.02,RGR-sd,kT-0.02,RGR+sd,length=0))

with(subset(COMPsummary,spec=="pul" & treatm=="mix"),points(kT,RGR,,pch=19,col="gray40",cex=1.6))
with(subset(COMPsummary,spec=="pul" & treatm=="mix"),lines(kT,RGR,lty=1))
with(subset(COMPsummary,spec=="pul" & treatm=="mix"),arrows(kT,RGR-sd,kT,RGR+sd,length=0))

with(subset(LABsummary,spec=="pul" & temp!=19.5),points(kT+0.02,RGR,pch=1,cex=1.6,col="black"))
with(subset(LABMEANsummary,spec=="pul"),abline(lm(RGR~kT),lty=2,col="gray40"))
with(subset(LABsummary,spec=="pul" & temp!=19.5),arrows(kT+0.02,RGR-sd,kT+0.02,RGR+sd,length=0))
legend("bottomright",cex=1.3,pch=c(19,19,1),col=c("gray40","black","black"),c("Interspecific", "Intraspecific", "Individual"),bty="n")
legend("topleft",c("C"),bty="n", cex=1.6, adj=c(2,-0.2))
mtext(expression(italic("C. pulchellum")),3,padj=-1.5)
axis(3,expression(paste("16",degree,"C"),paste("21",degree,"C")),at=c(-40.0916214241359, -39.3840822033468))



############################################################################################################################
######## TABLE 1 "##########################################################################################################
############################################################################################################################
merged = laboutput                                            #Merge the data from both experiment for statistical testing
merged = merged[,c("RGR","spec","temp","fam","pop")]
names(merged) = c("RGR","spec","temp","id.spec","id")
merged$treatm = factor(c("ind"))
merged = rbind(merged,compoutput[,c("RGR","spec","temp","id.spec","id","treatm")])
names(merged) = c("RGR","spec","temp","batch","pop","treatm")
merged$temp[merged$temp=="lo"] = as.numeric(c(16.3))
merged$temp[merged$temp=="hi"] = as.numeric(c(21.5))
merged$temp = as.numeric(merged$temp)


merged.mean.3temp = aggregate(merged[c("RGR")],by=merged[c("treatm","temp","spec","batch")],FUN=mean)    #Create one dataset that includes all three temperatures in the common garden experimetn
merged.mean.3temp$logRGR = log(merged.mean.3temp$RGR)
merged.mean.3temp$Kelvin<-merged.mean.3temp$temp+273.15
merged.mean.3temp$kT<- -1/(8.617332e-05*merged.mean.3temp$Kelvin)
merged.mean.3temp$kTF = as.factor(merged.mean.3temp$kT)            #Temperature as a factor (for Tukey's tests)

merged.mean.2temp = subset(merged.mean.3temp,temp!=19.5)           #One dataset  with only 16 and 21 degrees, for comparing the two experimens
merged.mean.2temp$kTF = as.factor(merged.mean.2temp$kT)            #Temperature as a factor (for Tukey's tests)

#Table 1A
m1A = lm(logRGR~spec*kT,data=merged.mean.3temp[merged.mean.3temp$treatm=="ind",])
anova(m1A)
AIC(m1A)

#Table 1B
m1B = lm(logRGR~spec*kT*treatm,data=merged.mean.2temp[merged.mean.2temp$treatm!="ind",])
anova(m1B)

#Table 1C
m1C = lm(logRGR~kT*treatm,data=merged.mean.2temp[merged.mean.2temp$spec=="arm",])
anova(m1C)
  #Tukey's HSD test
  m1C = aov(logRGR~kTF+treatm,data=merged.mean.2temp[merged.mean.2temp$spec=="arm",])
  TukeyHSD(m1C)

#Table 1D
m1D = lm(logRGR~kT*treatm,data=merged.mean.2temp[merged.mean.2temp$spec=="pul",])
anova(m1D)
  m1D = lm(logRGR~kTF*treatm,data=merged.mean.2temp[merged.mean.2temp$spec=="pul",])
  testInteractions(m1D)


